@@ -11,16 +11,16 @@ use Dancer::Config;
 use Dancer::ModuleLoader;
 use Dancer::SharedData;
 use Dancer::Logger;
+use Dancer::Exception qw(:all);
 
 sub new {
     my $class = shift;
 
-    croak "Plack::Request is needed by the PSGI handler"
+    raise core_handler_PSGI => "Plack::Request is needed by the PSGI handler"
       unless Dancer::ModuleLoader->load('Plack::Request');
 
     my $self = {};
     bless $self, $class;
-    return $self;
 }
 
 sub start {
@@ -43,7 +43,7 @@ sub apply_plack_middlewares_map {
     my $mw_map = Dancer::Config::setting('plack_middlewares_map');
 
     foreach my $req (qw(Plack::App::URLMap Plack::Builder)) {
-        croak "$req is needed to use apply_plack_middlewares_map"
+        raise core_handler_PSGI => "$req is needed to use apply_plack_middlewares_map"
           unless Dancer::ModuleLoader->load($req);
     }
 
@@ -51,7 +51,7 @@ sub apply_plack_middlewares_map {
 
     while ( my ( $path, $mw ) = each %$mw_map ) {
         my $builder = Plack::Builder->new();
-        map { $builder->add_middleware(@$_) } @$mw;
+        $builder->add_middleware(@$_) for @$mw;
         $urlmap->map( $path => $builder->to_app($app) );
     }
 
@@ -64,22 +64,20 @@ sub apply_plack_middlewares {
 
     my $middlewares = Dancer::Config::setting('plack_middlewares');
 
-    croak "Plack::Builder is needed for middlewares support"
+    raise core_handler_PSGI => "Plack::Builder is needed for middlewares support"
       unless Dancer::ModuleLoader->load('Plack::Builder');
 
+    ref $middlewares eq "ARRAY"
+      or raise core_handler_PSGI => "'plack_middlewares' setting must be an ArrayRef";
+
     my $builder = Plack::Builder->new();
 
-    ref $middlewares eq "ARRAY"
-      or croak "'plack_middlewares' setting must be an ArrayRef";
+    for my $mw (@$middlewares) {
+        Dancer::Logger::core "add middleware " . $mw->[0];
+        $builder->add_middleware(@$mw)
+    }
 
-    map {
-        Dancer::Logger::core "add middleware " . $_->[0];
-        $builder->add_middleware(@$_)
-    } @$middlewares;
-
-    $app = $builder->to_app($app);
-
-    return $app;
+    return $builder->to_app($app);
 }
 
 sub init_request_headers {
@@ -89,3 +87,33 @@ sub init_request_headers {
 }
 
 1;
+__END__
+
+=pod
+
+=head1 NAME
+
+Dancer::Handler::PSGI - a PSGI handler for Dancer applications
+
+=head1 DESCRIPTION
+
+This handler allows Dancer applications to run as part of PSGI stacks. Dancer
+will automatically determine when running in a PSGI environment and enable this
+handler, such that calling C<dance> will return a valid PSGI application.
+
+You may enable Plack middleware in your configuration file under the
+C<plack_middlewares> key. See L<Dancer::Cookbook> for more information.
+
+Note that you must have L<Plack> installed for this handler to work.
+
+=head1 USAGE
+    
+    # in bin/app.pl
+    set apphandler => 'Debug';
+
+    # then, run the app the following way
+    perl -d bin/app.pl GET '/some/path/to/test' 'with=parameters&other=42'
+
+=head1 AUTHORS
+
+Dancer contributors
