@@ -12,8 +12,9 @@ use Dancer::MIME;
 use HTTP::Headers;
 use Dancer::SharedData;
 use Dancer::Exception qw(:all);
+use Dancer::Continuation::Halted;
 
-__PACKAGE__->attributes(qw/content pass/);
+__PACKAGE__->attributes(qw/content pass streamed/);
 
 # constructor
 sub init {
@@ -101,7 +102,6 @@ sub halt {
             halted => 1,
         );
     }
-    raise E_HALTED;
 }
 
 sub halted {
@@ -143,6 +143,9 @@ sub headers {
 sub headers_to_array {
     my $self = shift;
 
+    # Time to finalise cookie headers, now
+    $self->build_cookie_headers;
+
     my $headers = [
         map {
             my $k = $_;
@@ -157,8 +160,34 @@ sub headers_to_array {
     return $headers;
 }
 
+# Given a cookie name and object, add it to the cookies we're going to send.
+# Stores them in a hashref within the response object until the response is
+# being built, so that, if the same cookie is set multiple times, only the last
+# value given to it will appear in a Set-Cookie header.
+sub add_cookie {
+    my ($self, $name, $cookie) = @_;
+    if ($self->{_built_cookies}) {
+        die "Too late to set another cookie, headers already built";
+    }
+    $self->{_cookies}{$name} = $cookie;
+}
+
+
+# When the response is about to be rendered, that's when we build up the
+# Set-Cookie headers
+sub build_cookie_headers {
+    my $self = shift;
+    for my $name (keys %{ $self->{_cookies} }) {
+        my $header = $self->{_cookies}{$name}->to_header;
+        $self->push_header(
+            'Set-Cookie' => $header,
+        );
+    }
+    $self->{_built_cookies}++;
+}
 1;
 
+
 =head1 NAME
 
 Dancer::Response - Response object for Dancer
@@ -189,7 +218,7 @@ Dancer::Response - Response object for Dancer
     Dancer::Response->new(
         status  => 200,
         content => 'my content',
-        headers => HTTP::Headers->new(...),
+        headers => ['X-Foo' => 'foo-value', 'X-Bar' => 'bar-value'],
     );
 
 create and return a new L<Dancer::Response> object
@@ -293,8 +322,8 @@ get or set the value of a header
 
 =head2 headers
 
-    $response->headers(HTTP::Headers->new(...));
-    Dancer::SharedData->response->headers(HTTP::Headers->new(...));
+    $response->headers('X-Foo' => 'fff', 'X-Bar' => 'bbb');
+    Dancer::SharedData->response->headers('X-Foo' => 'fff', 'X-Bar' => 'bbb');
 
 return the list of headers for the current response
 
